@@ -1,4 +1,4 @@
-/*
+/* vi:set et sw=2 sts=2 cin cino=t0,f0,(0,{s,>2s,n-s,^-s,e-s:
  * Copyright © 2014 Red Hat, Inc
  *
  * This program is free software; you can redistribute it and/or
@@ -27,7 +27,7 @@
 
 #include <glib/gi18n.h>
 
-#include "libglnx/libglnx.h"
+#include "libglnx.h"
 
 #include "flatpak-builtins.h"
 #include "flatpak-builtins-utils.h"
@@ -69,6 +69,8 @@ typedef struct
 {
   FlatpakDir *dir;
   GHashTable *refs_hash;
+  GHashTable *runtime_app_map;
+  GHashTable *extension_app_map;
   GPtrArray  *refs;
 } UninstallDir;
 
@@ -89,18 +91,28 @@ uninstall_dir_free (UninstallDir *udir)
 {
   g_object_unref (udir->dir);
   g_hash_table_unref (udir->refs_hash);
+  g_clear_pointer (&udir->runtime_app_map, g_hash_table_unref);
+  g_clear_pointer (&udir->extension_app_map, g_hash_table_unref);
   g_ptr_array_unref (udir->refs);
   g_free (udir);
 }
 
 static void
 uninstall_dir_add_ref (UninstallDir *udir,
-                       FlatpakDecomposed*ref)
+                       FlatpakDecomposed *ref)
 {
   if (g_hash_table_insert (udir->refs_hash, flatpak_decomposed_ref (ref), NULL))
     g_ptr_array_add (udir->refs, flatpak_decomposed_ref (ref));
 }
 
+static void
+uninstall_dir_remove_ref (UninstallDir *udir,
+                          FlatpakDecomposed *ref)
+{
+  g_hash_table_remove (udir->refs_hash, ref);
+  g_ptr_array_remove (udir->refs, ref);
+}
+
 static UninstallDir *
 uninstall_dir_ensure (GHashTable *uninstall_dirs,
                       FlatpakDir *dir)
@@ -141,6 +153,94 @@ flatpak_delete_data (gboolean    yes_opt,
   return TRUE;
 }
 
+static gboolean
+confirm_runtime_removal (gboolean           yes_opt,
+                         UninstallDir      *udir,
+                         FlatpakDecomposed *ref)
+{
+  g_autoptr(GPtrArray) apps = NULL;
+  g_autoptr(GError) local_error = NULL;
+  g_autofree char *ref_name = NULL;
+  const char *ref_branch;
+  const char *on = "";
+  const char *off = "";
+  gboolean is_extension;
+
+  if (flatpak_fancy_output ())
+    {
+      on = FLATPAK_ANSI_BOLD_ON;
+      off = FLATPAK_ANSI_BOLD_OFF;
+    }
+
+  is_extension = flatpak_dir_is_runtime_extension (udir->dir, ref);
+  if (is_extension)
+    {
+      apps = flatpak_dir_list_app_refs_with_runtime_extension (udir->dir,
+                                                               &udir->runtime_app_map,
+                                                               &udir->extension_app_map,
+                                                               ref, NULL, &local_error);
+      if (apps == NULL)
+        g_debug ("Unable to list apps using extension %s: %s\n",
+                 flatpak_decomposed_get_ref (ref), local_error->message);
+    }
+  else
+    {
+      apps = flatpak_dir_list_app_refs_with_runtime (udir->dir,
+                                                     &udir->runtime_app_map,
+                                                     ref, NULL, &local_error);
+      if (apps == NULL)
+        g_debug ("Unable to list apps using runtime %s: %s\n",
+                 flatpak_decomposed_get_ref (ref), local_error->message);
+    }
+
+  if (apps == NULL || apps->len == 0)
+    return TRUE;
+
+  /* Exclude any apps that will be removed by the current transaction */
+  for (guint i = 0; i < udir->refs->len; i++)
+    {
+      FlatpakDecomposed *uninstall_ref = g_ptr_array_index (udir->refs, i);
+      guint j;
+
+      if (flatpak_decomposed_is_runtime (uninstall_ref))
+        continue;
+
+      if (g_ptr_array_find_with_equal_func (apps, uninstall_ref,
+                                            (GEqualFunc)flatpak_decomposed_equal, &j))
+        g_ptr_array_remove_index_fast (apps, j);
+    }
+
+  if (apps->len == 0)
+    return TRUE;
+
+  ref_name = flatpak_decomposed_dup_id (ref);
+  ref_branch = flatpak_decomposed_get_branch (ref);
+
+  if (is_extension)
+    g_print (_("Info: applications using the extension %s%s%s branch %s%s%s:\n"),
+             on, ref_name, off, on, ref_branch, off);
+  else
+    g_print (_("Info: applications using the runtime %s%s%s branch %s%s%s:\n"),
+             on, ref_name, off, on, ref_branch, off);
+
+  g_print ("   ");
+  for (guint i = 0; i < apps->len; i++)
+    {
+      FlatpakDecomposed *app_ref = g_ptr_array_index (apps, i);
+      g_autofree char *id = flatpak_decomposed_dup_id (app_ref);
+      if (i != 0)
+        g_print (", ");
+      g_print ("%s", id);
+    }
+  g_print ("\n");
+
+  if (!yes_opt &&
+      !flatpak_yes_no_prompt (FALSE, _("Really remove?")))
+    return FALSE;
+
+  return TRUE;
+}
+
 gboolean
 flatpak_builtin_uninstall (int argc, char **argv, GCancellable *cancellable, GError **error)
 {
@@ -152,7 +252,7 @@ flatpak_builtin_uninstall (int argc, char **argv, GCancellable *cancellable, GEr
   FlatpakKinds kinds;
   g_autoptr(GHashTable) uninstall_dirs = NULL;
 
-  context = g_option_context_new (_("[REF…] - Uninstall an application"));
+  context = g_option_context_new (_("[REF…] - Uninstall applications or runtimes"));
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
 
   if (!flatpak_option_context_parse (context, options, &argc, &argv,
@@ -169,6 +269,9 @@ flatpak_builtin_uninstall (int argc, char **argv, GCancellable *cancellable, GEr
   if (argc >= 2 && opt_unused)
     return usage_error (context, _("Must not specify REFs when using --unused"), error);
 
+  if (opt_noninteractive)
+    opt_yes = TRUE; /* Implied */
+
   prefs = &argv[1];
   n_prefs = argc - 1;
 
@@ -282,15 +385,30 @@ flatpak_builtin_uninstall (int argc, char **argv, GCancellable *cancellable, GEr
           UninstallDir *udir = NULL;
           gboolean found_exact_name_match = FALSE;
           g_autoptr(GPtrArray) chosen_pairs = NULL;
+          FindMatchingRefsFlags matching_refs_flags;
 
           pref = prefs[j];
 
-          flatpak_split_partial_ref_arg_novalidate (pref, kinds, opt_arch, default_branch,
-                                                    &matched_kinds, &match_id, &match_arch, &match_branch);
+          if (!flatpak_allow_fuzzy_matching (pref))
+            matching_refs_flags = FIND_MATCHING_REFS_FLAGS_NONE;
+          else
+            matching_refs_flags = FIND_MATCHING_REFS_FLAGS_FUZZY;
 
-          /* We used _novalidate so that the id can be partial, but we can still validate the branch */
-          if (match_branch != NULL && !flatpak_is_valid_branch (match_branch, -1, &local_error))
-            return flatpak_fail_error (error, FLATPAK_ERROR_INVALID_REF, _("Invalid branch %s: %s"), match_branch, local_error->message);
+          if (matching_refs_flags & FIND_MATCHING_REFS_FLAGS_FUZZY)
+            {
+              flatpak_split_partial_ref_arg_novalidate (pref, kinds, opt_arch, default_branch,
+                                                        &matched_kinds, &match_id, &match_arch, &match_branch);
+
+              /* We used _novalidate so that the id can be partial, but we can still validate the branch */
+              if (match_branch != NULL && !flatpak_is_valid_branch (match_branch, -1, &local_error))
+                return flatpak_fail_error (error, FLATPAK_ERROR_INVALID_REF,
+                                           _("Invalid branch %s: %s"), match_branch, local_error->message);
+            }
+          else if (!flatpak_split_partial_ref_arg (pref, kinds, opt_arch, default_branch,
+                                                   &matched_kinds, &match_id, &match_arch, &match_branch, error))
+            {
+              return FALSE;
+            }
 
           ref_dir_pairs = g_ptr_array_new_with_free_func ((GDestroyNotify) ref_dir_pair_free);
           for (k = 0; k < dirs->len; k++)
@@ -299,7 +417,7 @@ flatpak_builtin_uninstall (int argc, char **argv, GCancellable *cancellable, GEr
               g_autoptr(GPtrArray)  refs = NULL;
 
               refs = flatpak_dir_find_installed_refs (dir, match_id, match_branch, match_arch, kinds,
-                                                      FIND_MATCHING_REFS_FLAGS_FUZZY, error);
+                                                      matching_refs_flags, error);
               if (refs == NULL)
                 return FALSE;
               else if (refs->len == 0)
@@ -322,11 +440,16 @@ flatpak_builtin_uninstall (int argc, char **argv, GCancellable *cancellable, GEr
             {
               if (n_prefs == 1)
                 {
-                  g_set_error (error, FLATPAK_ERROR, FLATPAK_ERROR_NOT_INSTALLED,
-                               _("%s/%s/%s not installed"),
-                               match_id ? match_id : "*unspecified*",
-                               match_arch ? match_arch : "*unspecified*",
-                               match_branch ? match_branch : "*unspecified*");
+                  g_autoptr(GString) err_str = g_string_new ("");
+                  g_string_append_printf (err_str, _("No installed refs found for ‘%s’"), match_id);
+
+                  if (match_arch)
+                    g_string_append_printf (err_str, _(" with arch ‘%s’"), match_arch);
+                  if (match_branch)
+                    g_string_append_printf (err_str, _(" with branch ‘%s’"), match_branch);
+
+                  g_set_error_literal (error, FLATPAK_ERROR, FLATPAK_ERROR_NOT_INSTALLED,
+                                       err_str->str);
                   return FALSE;
                 }
 
@@ -361,6 +484,13 @@ flatpak_builtin_uninstall (int argc, char **argv, GCancellable *cancellable, GEr
         }
     }
 
+  if (n_prefs > 0 && g_hash_table_size (uninstall_dirs) == 0)
+    {
+      g_set_error (error, FLATPAK_ERROR, FLATPAK_ERROR_NOT_INSTALLED,
+                   _("None of the specified refs are installed"));
+      return FALSE;
+    }
+
   GLNX_HASH_TABLE_FOREACH_V (uninstall_dirs, UninstallDir *, udir)
   {
     g_autoptr(FlatpakTransaction) transaction = NULL;
@@ -379,14 +509,32 @@ flatpak_builtin_uninstall (int argc, char **argv, GCancellable *cancellable, GEr
     /* This disables the remote metadata update, since uninstall is a local-only op */
     flatpak_transaction_set_no_pull (transaction, TRUE);
 
-    for (i = 0; i < udir->refs->len; i++)
+    /* Walk through the array backwards so we can safely remove */
+    for (i = udir->refs->len; i > 0; i--)
       {
-        FlatpakDecomposed *ref = g_ptr_array_index (udir->refs, i);
+        FlatpakDecomposed *ref = g_ptr_array_index (udir->refs, i - 1);
+
+         /* In case it's a runtime for an installed app or an optional runtime
+          * extension of an installed app, prompt the user for confirmation (in
+          * the former case the transaction will error out if executed).  This
+          * is limited to checking within the same installation; it won't
+          * prompt for a user app depending on a system runtime.
+         */
+        if (!opt_force_remove &&
+            !confirm_runtime_removal (opt_yes, udir, ref))
+          {
+            uninstall_dir_remove_ref (udir, ref);
+            continue;
+          }
 
         if (!flatpak_transaction_add_uninstall (transaction, flatpak_decomposed_get_ref (ref), error))
           return FALSE;
       }
 
+    /* These caches may no longer be valid once the transaction runs */
+    g_clear_pointer (&udir->runtime_app_map, g_hash_table_unref);
+    g_clear_pointer (&udir->extension_app_map, g_hash_table_unref);
+
     if (!flatpak_transaction_run (transaction, cancellable, error))
       {
         if (g_error_matches (*error, FLATPAK_ERROR, FLATPAK_ERROR_ABORTED))
