@@ -1,4 +1,4 @@
-/*
+/* vi:set et sw=2 sts=2 cin cino=t0,f0,(0,{s,>2s,n-s,^-s,e-s:
  * Copyright © 2014-2019 Red Hat, Inc
  *
  * This program is free software; you can redistribute it and/or
@@ -38,11 +38,10 @@
 #include <glib/gi18n-lib.h>
 
 #include <gio/gio.h>
-#include "libglnx/libglnx.h"
+#include "libglnx.h"
 
 #include "flatpak-exports-private.h"
 #include "flatpak-run-private.h"
-#include "flatpak-proxy.h"
 #include "flatpak-utils-base-private.h"
 #include "flatpak-dir-private.h"
 #include "flatpak-systemd-dbus-generated.h"
@@ -53,7 +52,15 @@
    flatpak_abs_usrmerged_dirs get the same treatment without having to be listed
    here. */
 const char *dont_export_in[] = {
-  "/usr", "/etc", "/app", "/dev", "/proc", NULL
+  "/.flatpak-info",
+  "/app",
+  "/dev",
+  "/etc",
+  "/proc",
+  "/run/flatpak",
+  "/run/host",
+  "/usr",
+  NULL
 };
 
 static char *
@@ -94,6 +101,40 @@ is_export_mode (int mode)
           || mode == FAKE_MODE_SYMLINK);
 }
 
+static inline const char *
+export_mode_to_verb (int mode)
+{
+  switch (mode)
+    {
+      case FAKE_MODE_DIR:
+        return "ensure existence of directory";
+
+      case FAKE_MODE_SYMLINK:
+        return "create symbolic link";
+
+      default:
+        break;
+    }
+
+  switch ((FlatpakFilesystemMode) mode)
+    {
+      case FLATPAK_FILESYSTEM_MODE_READ_ONLY:
+        return "export read-only";
+
+      case FLATPAK_FILESYSTEM_MODE_CREATE:
+        return "create and export read/write";
+
+      case FLATPAK_FILESYSTEM_MODE_READ_WRITE:
+        return "export read/write";
+
+      case FLATPAK_FILESYSTEM_MODE_NONE:
+        return "replace with tmpfs";
+
+      default:
+        return "[use unknown/invalid mode?]";
+    }
+}
+
 typedef struct
 {
   char *path;
@@ -395,6 +436,8 @@ flatpak_exports_append_bwrap_args (FlatpakExports *exports,
 
   g_qsort_with_data (keys, n_keys, sizeof (char *), (GCompareDataFunc) flatpak_strcmp0_ptr, NULL);
 
+  flatpak_debug2 ("Converting FlatpakExports to bwrap arguments...");
+
   for (l = eps; l != NULL; l = l->next)
     {
       ExportedPath *ep = l->data;
@@ -404,7 +447,14 @@ flatpak_exports_append_bwrap_args (FlatpakExports *exports,
 
       if (ep->mode == FAKE_MODE_SYMLINK)
         {
-          if (!path_parent_is_mapped (keys, n_keys, exports->hash, path))
+          flatpak_debug2 ("\"%s\" is meant to be a symlink", path);
+
+          if (path_parent_is_mapped (keys, n_keys, exports->hash, path))
+            {
+              flatpak_debug2 ("Not creating \"%s\" as symlink because its parent is "
+                              "already mapped", path);
+            }
+          else
             {
               g_autofree char *resolved = flatpak_exports_resolve_link_in_host (exports,
                                                                                 path,
@@ -413,30 +463,60 @@ flatpak_exports_append_bwrap_args (FlatpakExports *exports,
                 {
                   g_autofree char *parent = g_path_get_dirname (path);
                   g_autofree char *relative = make_relative (parent, resolved);
+
+                  flatpak_debug2 ("Resolved \"%s\" to \"%s\" in host", path, resolved);
+                  flatpak_debug2 ("Creating \"%s\" -> \"%s\" in sandbox", path, relative);
                   flatpak_bwrap_add_args (bwrap, "--symlink", relative, path,  NULL);
                 }
+              else
+                {
+                  flatpak_debug2 ("Unable to resolve \"%s\" in host, skipping", path);
+                }
             }
         }
       else if (ep->mode == FAKE_MODE_TMPFS)
         {
+          flatpak_debug2 ("\"%s\" is meant to be a tmpfs or empty directory", path);
+
           /* Mount a tmpfs to hide the subdirectory, but only if there
              is a pre-existing dir we can mount the path on. */
           if (path_is_dir (exports, path))
             {
               if (!path_parent_is_mapped (keys, n_keys, exports->hash, path))
                 /* If the parent is not mapped, it will be a tmpfs, no need to mount another one */
-                flatpak_bwrap_add_args (bwrap, "--dir", path, NULL);
+                {
+                  flatpak_debug2 ("Parent of \"%s\" is not mapped, creating empty directory", path);
+                  flatpak_bwrap_add_args (bwrap, "--dir", path, NULL);
+                }
               else
-                flatpak_bwrap_add_args (bwrap, "--tmpfs", path, NULL);
+                {
+                  flatpak_debug2 ("Parent of \"%s\" is mapped, creating tmpfs to shadow it", path);
+                  flatpak_bwrap_add_args (bwrap, "--tmpfs", path, NULL);
+                }
+            }
+          else
+            {
+              flatpak_debug2 ("Not a directory, skipping: \"%s\"", path);
             }
         }
       else if (ep->mode == FAKE_MODE_DIR)
         {
+          flatpak_debug2 ("\"%s\" is meant to be a directory", path);
+
           if (path_is_dir (exports, path))
-            flatpak_bwrap_add_args (bwrap, "--dir", path, NULL);
+            {
+              flatpak_debug2 ("Ensuring \"%s\" is created as a directory", path);
+              flatpak_bwrap_add_args (bwrap, "--dir", path, NULL);
+            }
+          else
+            {
+              flatpak_debug2 ("Not a directory, skipping: \"%s\"", path);
+            }
         }
       else
         {
+          flatpak_debug2 ("\"%s\" is meant to be shared (ro or rw) with the container",
+                          path);
           flatpak_bwrap_add_args (bwrap,
                                   (ep->mode == FLATPAK_FILESYSTEM_MODE_READ_ONLY) ? "--ro-bind" : "--bind",
                                   path, path, NULL);
@@ -680,9 +760,29 @@ do_export_path (FlatpakExports *exports,
   ep->path = g_strdup (path);
 
   if (old_ep != NULL)
-    ep->mode = MAX (old_ep->mode, mode);
+    {
+      if (old_ep->mode < mode)
+        {
+          flatpak_debug2 ("Increasing export mode from \"%s\" to \"%s\": %s",
+                          export_mode_to_verb (old_ep->mode),
+                          export_mode_to_verb (mode),
+                          path);
+          ep->mode = mode;
+        }
+      else
+        {
+          flatpak_debug2 ("Not changing export mode from \"%s\" to \"%s\": %s",
+                          export_mode_to_verb (old_ep->mode),
+                          export_mode_to_verb (mode),
+                          path);
+          ep->mode = old_ep->mode;
+        }
+    }
   else
-    ep->mode = mode;
+    {
+      flatpak_debug2 ("Will %s: %s", export_mode_to_verb (mode), path);
+      ep->mode = mode;
+    }
 
   g_hash_table_replace (exports->hash, ep->path, ep);
 }
@@ -769,12 +869,18 @@ check_if_autofs_works (FlatpakExports *exports,
   return TRUE;
 }
 
-/* We use level to avoid infinite recursion */
+/* We use level to avoid infinite recursion.
+ *
+ * Note that some of the errors produced by this function are "real errors"
+ * and should show up as a user-visible warning, but others are relatively
+ * uninteresting, and in general none are actually fatal: we prefer to
+ * continue with fewer paths exposed rather than failing to run. */
 static gboolean
 _exports_path_expose (FlatpakExports *exports,
                       int             mode,
                       const char     *path,
-                      int             level)
+                      int             level,
+                      GError        **error)
 {
   g_autofree char *canonical = NULL;
   struct stat st;
@@ -785,36 +891,59 @@ _exports_path_expose (FlatpakExports *exports,
 
   g_return_val_if_fail (is_export_mode (mode), FALSE);
 
+  flatpak_debug2 ("Trying to %s: %s", export_mode_to_verb (mode), path);
+
   if (level > 40) /* 40 is the current kernel ELOOP check */
     {
-      g_debug ("Expose too deep, bail");
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_TOO_MANY_LINKS,
+                   "%s", g_strerror (ELOOP));
       return FALSE;
     }
 
   if (!g_path_is_absolute (path))
     {
-      g_debug ("Not exposing relative path %s", path);
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_FILENAME,
+                   _("An absolute path is required"));
       return FALSE;
     }
 
   /* Check if it exists at all */
   o_path_fd = flatpak_exports_open_in_host (exports, path, O_PATH | O_NOFOLLOW);
+
   if (o_path_fd == -1)
-    return FALSE;
+    {
+      int saved_errno = errno;
+
+      /* Intentionally using G_IO_ERROR_NOT_FOUND even if errno is
+       * something different, so callers can suppress the warning in this
+       * relatively likely and uninteresting case: we don't particularly
+       * care whether this is happening as a result of ENOENT or EACCES
+       * or any other reason. */
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                   _("Unable to open path \"%s\": %s"),
+                   path, g_strerror (saved_errno));
+      return FALSE;
+    }
 
   if (fstat (o_path_fd, &st) != 0)
-    return FALSE;
+    return glnx_throw (error,
+                       _("Unable to get file type of \"%s\": %s"),
+                       path, g_strerror (errno));
 
   /* Don't expose weird things */
   if (!(S_ISDIR (st.st_mode) ||
         S_ISREG (st.st_mode) ||
         S_ISLNK (st.st_mode) ||
         S_ISSOCK (st.st_mode)))
-    return FALSE;
+    return glnx_throw (error,
+                       _("File \"%s\" has unsupported type 0o%o"),
+                       path, st.st_mode & S_IFMT);
 
   /* O_PATH + fstatfs is the magic that we need to statfs without automounting the target */
   if (fstatfs (o_path_fd, &stfs) != 0)
-    return FALSE;
+    return glnx_throw (error,
+                       _("Unable to get filesystem information for \"%s\": %s"),
+                       path, g_strerror (errno));
 
   if (stfs.f_type == AUTOFS_SUPER_MAGIC ||
       (G_UNLIKELY (exports->test_flags & FLATPAK_EXPORTS_TEST_FLAGS_AUTOFS) &&
@@ -822,7 +951,8 @@ _exports_path_expose (FlatpakExports *exports,
     {
       if (!check_if_autofs_works (exports, path))
         {
-          g_debug ("ignoring blocking autofs path %s", path);
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK,
+                       _("Ignoring blocking autofs path \"%s\""), path);
           return FALSE;
         }
     }
@@ -837,17 +967,35 @@ _exports_path_expose (FlatpakExports *exports,
          create the parents for them anyway */
       if (flatpak_has_path_prefix (path, dont_export_in[i]))
         {
-          g_debug ("skipping export for path %s", path);
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_MOUNTABLE_FILE,
+                       _("Path \"%s\" is reserved by Flatpak"),
+                       dont_export_in[i]);
+          return FALSE;
+        }
+
+      /* Also don't expose directories that are a parent of a directory
+       * that is "owned" by the sandboxing framework. For example, because
+       * Flatpak controls /run/host and /run/flatpak, we cannot allow
+       * --filesystem=/run, which would prevent us from creating the
+       * contents of /run/host and /run/flatpak. */
+      if (flatpak_has_path_prefix (dont_export_in[i], path))
+        {
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_MOUNTABLE_FILE,
+                       _("Path \"%s\" is reserved by Flatpak"),
+                       dont_export_in[i]);
           return FALSE;
         }
     }
 
   for (i = 0; flatpak_abs_usrmerged_dirs[i] != NULL; i++)
     {
-      /* Same as /usr, but for the directories that get merged into /usr */
+      /* Same as /usr, but for the directories that get merged into /usr.
+       * Keep the translatable string here the same as the one above */
       if (flatpak_has_path_prefix (path, flatpak_abs_usrmerged_dirs[i]))
         {
-          g_debug ("skipping export for path %s", path);
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_MOUNTABLE_FILE,
+                       _("Path \"%s\" is reserved by Flatpak"),
+                       flatpak_abs_usrmerged_dirs[i]);
           return FALSE;
         }
     }
@@ -861,27 +1009,51 @@ _exports_path_expose (FlatpakExports *exports,
       if (slash)
         *slash = 0;
 
-      if (path_is_symlink (exports, path) && !never_export_as_symlink (path))
+      if (!path_is_symlink (exports, path))
         {
-          g_autofree char *resolved = flatpak_exports_resolve_link_in_host (exports, path, NULL);
+          flatpak_debug2 ("%s is not a symlink", path);
+        }
+      else if (never_export_as_symlink (path))
+        {
+          flatpak_debug2 ("%s is a symlink, but we avoid exporting it as such", path);
+        }
+      else
+        {
+          g_autoptr(GError) local_error = NULL;
+          g_autofree char *resolved = flatpak_exports_resolve_link_in_host (exports, path, &local_error);
           g_autofree char *new_target = NULL;
 
           if (resolved)
             {
+              flatpak_debug2 ("%s is a symlink, resolved to %s", path, resolved);
+
               if (slash)
                 new_target = g_build_filename (resolved, slash + 1, NULL);
               else
                 new_target = g_strdup (resolved);
 
-              if (_exports_path_expose (exports, mode, new_target, level + 1))
+              flatpak_debug2 ("Trying to export the target instead: %s", new_target);
+
+              if (_exports_path_expose (exports, mode, new_target, level + 1, &local_error))
                 {
                   do_export_path (exports, path, FAKE_MODE_SYMLINK);
                   return TRUE;
                 }
-            }
 
-          return FALSE;
+              flatpak_debug2 ("Could not export target %s, so ignoring %s",
+                              new_target, path);
+              g_propagate_error (error, g_steal_pointer (&local_error));
+              return FALSE;
+            }
+          else
+            {
+              g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                           _("Unable to resolve symbolic link \"%s\": %s"),
+                           path, local_error->message);
+              return FALSE;
+            }
         }
+
       if (slash)
         *slash = '/';
     }
@@ -891,42 +1063,46 @@ _exports_path_expose (FlatpakExports *exports,
   return TRUE;
 }
 
-void
-flatpak_exports_add_path_expose (FlatpakExports       *exports,
-                                 FlatpakFilesystemMode mode,
-                                 const char           *path)
+gboolean
+flatpak_exports_add_path_expose (FlatpakExports         *exports,
+                                 FlatpakFilesystemMode   mode,
+                                 const char             *path,
+                                 GError                **error)
 {
-  g_return_if_fail (mode > FLATPAK_FILESYSTEM_MODE_NONE);
-  g_return_if_fail (mode <= FLATPAK_FILESYSTEM_MODE_LAST);
-  _exports_path_expose (exports, mode, path, 0);
+  g_return_val_if_fail (mode > FLATPAK_FILESYSTEM_MODE_NONE, FALSE);
+  g_return_val_if_fail (mode <= FLATPAK_FILESYSTEM_MODE_LAST, FALSE);
+  return _exports_path_expose (exports, mode, path, 0, error);
 }
 
-void
-flatpak_exports_add_path_tmpfs (FlatpakExports *exports,
-                                const char     *path)
+gboolean
+flatpak_exports_add_path_tmpfs (FlatpakExports  *exports,
+                                const char      *path,
+                                GError         **error)
 {
-  _exports_path_expose (exports, FAKE_MODE_TMPFS, path, 0);
+  return _exports_path_expose (exports, FAKE_MODE_TMPFS, path, 0, error);
 }
 
-void
-flatpak_exports_add_path_expose_or_hide (FlatpakExports       *exports,
-                                         FlatpakFilesystemMode mode,
-                                         const char           *path)
+gboolean
+flatpak_exports_add_path_expose_or_hide (FlatpakExports        *exports,
+                                         FlatpakFilesystemMode  mode,
+                                         const char            *path,
+                                         GError               **error)
 {
-  g_return_if_fail (mode >= FLATPAK_FILESYSTEM_MODE_NONE);
-  g_return_if_fail (mode <= FLATPAK_FILESYSTEM_MODE_LAST);
+  g_return_val_if_fail (mode >= FLATPAK_FILESYSTEM_MODE_NONE, FALSE);
+  g_return_val_if_fail (mode <= FLATPAK_FILESYSTEM_MODE_LAST, FALSE);
 
   if (mode == FLATPAK_FILESYSTEM_MODE_NONE)
-    flatpak_exports_add_path_tmpfs (exports, path);
+    return flatpak_exports_add_path_tmpfs (exports, path, error);
   else
-    flatpak_exports_add_path_expose (exports, mode, path);
+    return flatpak_exports_add_path_expose (exports, mode, path, error);
 }
 
-void
-flatpak_exports_add_path_dir (FlatpakExports *exports,
-                              const char     *path)
+gboolean
+flatpak_exports_add_path_dir (FlatpakExports  *exports,
+                              const char      *path,
+                              GError         **error)
 {
-  _exports_path_expose (exports, FAKE_MODE_DIR, path, 0);
+  return _exports_path_expose (exports, FAKE_MODE_DIR, path, 0, error);
 }
 
 void
