@@ -1,4 +1,4 @@
-/*
+/* vi:set et sw=2 sts=2 cin cino=t0,f0,(0,{s,>2s,n-s,^-s,e-s:
  * Copyright © 2021 Red Hat, Inc
  *
  * This program is free software; you can redistribute it and/or
@@ -35,7 +35,7 @@
 #include "flatpak-error.h"
 #include "flatpak-prune-private.h"
 #include "flatpak-variant-impl-private.h"
-#include "libglnx/libglnx.h"
+#include "libglnx.h"
 #include "valgrind-private.h"
 
 /* This is a custom implementation of ostree-prune that caches the
